/**** InstantSQL.java **** Jerry Smith, jsmith@spectro.com ****/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.URL;
import java.sql.*;

public class InstantSQL extends Frame implements ActionListener, WindowListener {
	//
	// Constants:
	//
	private static final String URL_PROTOCOL = "jdbc:idb:";
	private static final String PROP_FILE = "XXX.prp";
	private static final int URL_WIDTH = 40;
	private static final int MESSAGE_WIDTH = 70;
	private static final int SQL_WIDTH = 80;
	private static final int RESULT_WIDTH = 80;

	//
	// Instance variables:
	//
	private Button connect, submit, clear;
	private TextField urlField;
	private TextArea sqlArea;
	private TextArea resultArea;
	private TextField messageField;
	private DatabaseMetaData dmd;
	private Connection con = null;
	private Statement stmt = null;

	public InstantSQL(String title, String propFile)
	{
		super(title);
		setLayout(new BorderLayout(5, 5));
		InsetsPanel outerPanel = new InsetsPanel();
		outerPanel.setLayout(new BorderLayout(5, 5));
		add(outerPanel, BorderLayout.CENTER);

		Panel urlPanel = new Panel();
		outerPanel.add(urlPanel, BorderLayout.NORTH);
		urlPanel.add(new Label("URL:"));
		String url = URL_PROTOCOL;
		url += (propFile == null || propFile.length() == 0) ? PROP_FILE : propFile;
		
		urlField = new TextField(url, URL_WIDTH);
		urlPanel.add(urlField);
		connect = new Button(" Connect ");
		connect.addActionListener(this);
		urlPanel.add(connect);

		Panel innerPanel = new Panel();
		innerPanel.setLayout(new BorderLayout(5, 5));
		outerPanel.add(innerPanel, BorderLayout.CENTER);
		Panel sqlPanel = new Panel();
		sqlPanel.setLayout(new BorderLayout(5, 5));
		innerPanel.add(sqlPanel, BorderLayout.NORTH);

		sqlPanel.add(new Label("SQL:", Label.CENTER), BorderLayout.NORTH);
		sqlArea = new TextArea(5, SQL_WIDTH);
		sqlPanel.add(sqlArea, BorderLayout.CENTER);
		Panel buttonPanel = new Panel();
		submit = new Button(" Submit ");
		submit.addActionListener(this);
		buttonPanel.add(submit);
		clear = new Button(" Clear ");
		clear.addActionListener(this);
		buttonPanel.add(clear);
		sqlPanel.add(buttonPanel, BorderLayout.SOUTH);

		Panel resultPanel = new Panel();
		resultPanel.setLayout(new BorderLayout(5, 5));
		innerPanel.add(resultPanel, BorderLayout.CENTER);

		resultPanel.add(new Label("Result:", Label.CENTER), BorderLayout.NORTH);
		resultArea = new TextArea(15, RESULT_WIDTH);
		resultPanel.add(resultArea, BorderLayout.CENTER);
		Panel messagePanel = new Panel();
		outerPanel.add(messagePanel, BorderLayout.SOUTH);
		messagePanel.add(new Label("Status:"));
		messageField = new TextField(MESSAGE_WIDTH);
		messagePanel.add(messageField);

		pack();
		setSize(getPreferredSize());
		setVisible(true);
		addWindowListener(this);
	}	 // InstantSQL //


	public InstantSQL(String title)
	{
		this(title, null);
	}	// InstantSQL //


	public static void main(String[] args)
	{
		if (args.length > 1) {
			System.out.println("Usage: java InstantSQL [<database-properties-file>]");
			return;
		}
			InstantSQL isql = (args.length == 1) ?
				new InstantSQL("InstantSQL for InstantDB", args[0]) :
				new InstantSQL("InstantSQL for InstantDB");
	}	 // main //


	private boolean connectToDB()
	{
		closeDB();
		try {
			Class.forName("jdbc.idbDriver").newInstance();
			con = DriverManager.getConnection(urlField.getText());
			stmt = con.createStatement();
			dmd = con.getMetaData();
			String result = "Connected to: " + dmd.getURL() + "\n" +
				"Driver: " + dmd.getDriverName() + "\n" +
				"Version: " + dmd.getDriverVersion();
			resultArea.setText(result);
			messageField.setText("Successfully connected to database.");
			return true;
		}
		catch (Exception e) {
			System.out.println("Error connecting to database: " +
				urlField.getText());
			messageField.setText("Error connecting to database: " +
				urlField.getText());
			return false;
		}
	}	 // connectToDB //


	private void closeDB()
	{
		try {
			if (con != null)
				con.close();
		}
		catch (Exception e) {
			System.out.println("Error closing the database connection: " +
				urlField.getText());
		}
	}	 // closeDB //


	private void closeStatement()
	{
		try {
			if (stmt != null)
				stmt.close();
		}
		catch (Exception e) {
			System.out.println("Error closing the current statement.");
		}
	}	 // closeStatement //


	private void displayResultSet(ResultSet rs)
	{
		String result = "";
		try {
			ResultSetMetaData rsmd = rs.getMetaData();
			int cols = rsmd.getColumnCount();
			for (int i = 1; i <= cols; i++) {
				if (i > 1)
					result += ", ";
				result += rsmd.getColumnLabel(i);
			}
			result += "\n\n";
			while (rs.next()) {
				for (int i = 1; i <= cols; i++) {
					if (i > 1)
						result += ", ";
					result += rs.getString(i);
				}
				result += "\n\n";
			}
			resultArea.setText(result);
		}
		catch (Exception e) {
			System.out.println("Error displaying the current statement.");
			messageField.setText("Error displaying the current statement.");
		}
	}	// displayResultSet //


	private void executeSQL()
	{
		if (con == null) {
			messageField.setText("Please connect to a database.");
			return;
		}
		if (sqlArea.getText().length() == 0) {
			messageField.setText("SQL statement area is empty.");
			return;
		}
		try {
			if (stmt.execute(sqlArea.getText())) {
				ResultSet rs = stmt.getResultSet();
				displayResultSet(rs);
				rs.close();
			}
			else {
				messageField.setText("No text output from this SQL command.");
			}
		}
		catch (Exception e) {
			System.out.println("Error executing the current statement.");
			messageField.setText("Error executing the current statement.");
		}
	}	 // executeSQL //


	public void actionPerformed(ActionEvent event)
	{
		Object source = event.getSource();
		if (source == connect) {
			connectToDB();
		}
		else if (source == submit) {
			executeSQL();
		}
		else if (source == clear) {
			sqlArea.setText("");
			resultArea.setText("");
			messageField.setText("");
		}		
	}	// actionPerformed //


	public void windowClosing(WindowEvent e)
	{
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			closeStatement();
			closeDB();
			System.exit(0);
		}
	}	 // windowClosing //

	public void windowClosed(WindowEvent e) {}

	public void windowDeactivated(WindowEvent e) {}

	public void windowActivated(WindowEvent e) {}

	public void windowDeiconified(WindowEvent e) {}

	public void windowIconified(WindowEvent e) {}

	public void windowOpened(WindowEvent e) {}
}	 // InstantSQL class //


class InsetsPanel extends Panel {
	public Insets getInsets()
	{
		return new Insets(5, 5, 5, 5);
	}	 // getInsets //
}	 // InsetsPanel class //

