//
// Msql client library for JAVA
// Copyright (C) 1995 Darryl Collins <darryl@minmet.uq.oz.au>
// This software is provided "as is" without any expressed or implied warranty.
//
// This package implements a client for the mSQL server designed and
// implemented by David Hughes <bambi@hughes.com.au>
//
// Revision 1.2  1997/02/25
//

package msql;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;


/**
 * Object which contains a set of results returned from the server.  The results
 * might be a view which is the result of a SELECT query, or it may be a table
 * of information about a given database, table or field.  Note that the 
 * results of a SELECT query are never returned in a standalone instance of
 * MsqlData, but are always combined with a second instance of MsqlData
 * containing Field information.  The resulting combined data structure
 * is an instance of MsqlResult.
 */
class MsqlData {

	// Maximum number of rows it can cope with
	static int maxRows = 4096;
	
	// Number of rows and columns in table
	private int nRows=0, nCols=0;

	// Data storage
        private MsqlRow rowData[];

	// Cursor for data access
	private int nCursor=0; 

/**
 * Constructor - accepts number of fields as argument
 * 
 * @param n		the number of fields this instance will contain.
 */
	public MsqlData(int n) {
		nCols = n;
                rowData = new MsqlRow[maxRows];
	}

/**
 * Constructor - copies an existing object
 * 
 * @param d		another MsqlData to be duplicated.
 */
	public MsqlData(MsqlData d) {

		nRows = d.nRows;
		nCols = d.nCols;

                rowData = new MsqlRow[maxRows];

		for(int i=0; i<nRows; i++)
                        rowData[i] = new MsqlRow(d.rowData[i]);
	}

/** 
 * Add a new row to the table.  Accepts a raw string from the 
 * server as an argument and parses it to identify column contents
 *
 * @param s		a string obtained from the server which represents a row of the incoming table.
 */
        protected void addRow(String s) {

		int i,j,k,l;

		if(nRows >= maxRows) {
			try {
				MsqlRow newData[] = new MsqlRow[maxRows+1024];

				for (i=0; i<maxRows; i++) 
					newData[i] = rowData[i];

				maxRows += 1024;
				rowData = newData;
			} catch(Exception e) {
				return;
			}
		}

                rowData[nRows] = new MsqlRow(nCols, s);

		nRows++;
	}

/**
 * Return a specific row from the table
 * 
 * @param i		the index of the row to be retrieved.
 * @return		an array of Strings representing the columns of the row.
 */
        protected String [] getRow(int i) {
		
                return rowData[i].rowData();
	}

/**
 * Fetch the next row from the table
 *
 * @return		an array of Strings representing the columns of the row.
 */
	public String [] FetchRow() {

		if(nCursor < nRows)
                        return rowData[nCursor++].rowData();
		
		return null;
	}

/**
 * Data Seek
 *
 * @param i		the index of the row to be retrieved.
 */
	public void DataSeek(int i) {

		if(i < nRows)
			nCursor = i;
	}

/**
 * Return the number of fields in the table
 * 
 * @return		the number of fields (columns) in this data set.
 */
	public int NumFields() {
		
		return nCols;
	}

/**
 * return the number of rows in the table
 * 
 * @return		the number of rows in this data set.
 */
	public int NumRows() {

		return nRows;
	}
}
