/* UI applet to convert RGB to HSB, and vice versa */

/* Scrollbar Version by Ozgur Balsoy */
/* November 29, 1996                 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

//public class ColorsApplet extends Applet {
//
//   public void init() {
//
//    ColorFrame f = new ColorFrame();
//    f.resize(700, 200);
//    f.show();
//    }
//
// }

public class ColorFrame extends Frame {
  DrawCanvas parent;
  ColorFrame (DrawCanvas parent, Hashtable colors, Hashtable colorusage) {

    this.parent=parent;
    setLayout(new BorderLayout());

    ColorPanel cp=new ColorPanel();
    add(BorderLayout.NORTH, new ControlPanel(this, cp, colors, colorusage));
    add(BorderLayout.CENTER, cp);
    
  }
  public void sendColorsBack(Hashtable c, Hashtable cu) {
     parent.getNewColors(c, cu);
   } 
}

class ControlPanel extends Panel 
                implements ActionListener, ItemListener 
{
  ColorFrame parent;
  Choice where, colorchoice;
  ColorPanel cp;
  TextField colorname;
  Hashtable colors, colorusage;
  
  ControlPanel (ColorFrame _parent, ColorPanel _cp, 
                  Hashtable _colors, Hashtable _colorusage) {
   
    parent = _parent;
    cp = _cp;
    colors = _colors;
    colorusage = _colorusage;
    setLayout(new GridLayout(1, 5, 3, 3));

    where = new Choice();
    for (Enumeration e = colorusage.keys() ; e.hasMoreElements() ;) {
        where.addItem((String)e.nextElement()); }
    add(where);

    colorchoice = new Choice();
    for (Enumeration e = colors.keys() ; e.hasMoreElements() ;) {
        colorchoice.addItem((String)e.nextElement()); }
    add(colorchoice);

    Button b;
    add(b=new Button("Add color as .."));    
    b.addActionListener(this);
    add(colorname = new TextField());
    add(b=new Button("Update"));
    b.addActionListener(this);
    add(b=new Button("Quit"));
    b.addActionListener(this);
  }  

  public void changeColor(String newColor) {

    colorchoice.select(newColor);
    cp.changeColor((Color) colors.get(newColor));
    colorusage.put(where.getSelectedItem(), newColor);

  }  

  public void nameColor(String newColor) {  
    colors.put(newColor, cp.returnColor());
    colorchoice.addItem(newColor);
    changeColor(newColor);
  } 

  public void itemStateChanged(ItemEvent e) {
    
    String s = (String) e.getItem();
    if ((Choice)e.getItemSelectable() == where) {
      changeColor((String) colorusage.get(s));
    } else if ((Choice)e.getItemSelectable() == colorchoice) {
      changeColor(s);
    }
  }
  
  public void actionPerformed(ActionEvent e) {
    String s = e.getActionCommand();
    
    if (s.equals("Add color as ..")) {
      if (colorname.getText().equals("")) {
        colorname.setText("Name the color here.");
      } else { 
        nameColor(colorname.getText());
        colorname.setText("");
      }
    } else if (s.equals("Update")) {
      parent.sendColorsBack(colors, colorusage);
    } else if (s.equals("Quit")) {
      parent.dispose();
    }
  }
}

class ColorPanel extends Panel {
  ColorControls RGBcontrols, HSBcontrols;
  Canvas swatch;    

  ColorPanel() {
    Color theColor = new Color(0,0,0);
    float[] HSB = Color.RGBtoHSB(theColor.getRed(),theColor.getGreen(),
                               theColor.getBlue(), (new float[3]));
   
    setLayout(new GridLayout(1,3,10,10));
    
    // The color swatch
    swatch = new Canvas();
    swatch.setBackground(theColor);
    
    // the control panels
    RGBcontrols = new ColorControls(this,
          "Red", "Green", "Blue", 
           theColor.getRed(), theColor.getGreen(),theColor.getBlue());

    HSBcontrols = new ColorControls(this,
          "Hue", "Saturation", "Brightness", 
           (int)(HSB[0] * 360), (int)(HSB[1] * 100), (int)(HSB[2] * 100));
          
    add(swatch);
    add(RGBcontrols);
    add(HSBcontrols);

  }

  public Insets getInsets() {
    return new Insets(10,10,10,10);
  }

  public void changeColor(Color aColor) {
    RGBcontrols.setColor(aColor);
  }
  public Color returnColor() {
     return swatch.getBackground();
  }

  void update(ColorControls in) {
    Color c;

    if (in == RGBcontrols) {  // change to RGB
      c = new Color(in.r.getValue(),in.g.getValue(),in.b.getValue());
      swatch.setBackground(c);

      float[] HSB = Color.RGBtoHSB(c.getRed(),c.getGreen(),
                               c.getBlue(), (new float[3]));

      // In the original version, HSB values below are multiplied 
      // by 360, 100, and 100 respectively. Here they are multiplied
      // by 255 so that they fit into the scroll bar (min:0, max:255). 
      HSB[0] *= 255;
      HSB[1] *= 255;
      HSB[2] *= 255;
      HSBcontrols.r.setValue((int)HSB[0]);
      HSBcontrols.g.setValue((int)HSB[1]);
      HSBcontrols.b.setValue((int)HSB[2]);
    }
    else {  // change to HSB 
      int f1 = in.r.getValue();
      int f2 = in.g.getValue();
      int f3 = in.b.getValue();
      c = Color.getHSBColor((float)f1/255, (float)f2/255, (float)f3/255);
      swatch.setBackground(c);    
      RGBcontrols.r.setValue(c.getRed());
      RGBcontrols.g.setValue(c.getGreen());
      RGBcontrols.b.setValue(c.getBlue());

    }
    swatch.repaint();
  }
}

class ColorControls extends Panel 
                 implements AdjustmentListener 
{
  Scrollbar r, g, b;
  ColorPanel outerparent;

  ColorControls(ColorPanel target, String l1, String l2, String l3,
                int v1, int v2, int v3) {

    this.outerparent = target;
    setLayout(new GridLayout(3,2,10,10));
    
    r = new Scrollbar(Scrollbar.HORIZONTAL,v1,10,0,255);
    g = new Scrollbar(Scrollbar.HORIZONTAL,v2,10,0,255);
    b = new Scrollbar(Scrollbar.HORIZONTAL,v3,10,0,255);

    add(new Label(l1, Label.RIGHT));
    add(r);
    add(new Label(l2, Label.RIGHT));
    add(g);
    add(new Label(l3, Label.RIGHT));
    add(b);

  }
  public Insets getInsets() {
    return new Insets(10,10,0,0);
  }

  void setColor(Color aColor) {

    r.setValue(aColor.getRed());
    g.setValue(aColor.getGreen());
    b.setValue(aColor.getBlue());
    outerparent.update(this);

  }


  public void adjustmentValueChanged(AdjustmentEvent e) {
    
    this.outerparent.update(this);
  }
  
}


