// This source code is provided "as-is" and without any
// warranty of any kind, either express or implied.
//
// This code is placed in the public domain and may be
// freely modified or distributed as required.

import java.net.URL;
import java.sql.*;
import jdbc.idbDriver;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Properties;

class sample  {

	public static void main (String args[]) {

		try {

			// uncomment next line to enable DriverManager logging
			// java.sql.DriverManager.setLogStream(java.lang.System.out);


			// Create initial thread object to execute SQL from a file.

			String sqlfile = "sql1.txt";
			if (args.length > 0) {
				sqlfile = args[0];
			} // if

			Thread thread1 = new Thread (new SampleThread (sqlfile));

			// start the thread running

			thread1.start();

			// Wait for it to complete 

			thread1.join();

		}
		catch (Exception ex) {
			ex.printStackTrace ();
		}
	}
} // class sample

class SampleThread implements Runnable {
	
	Connection			con;
	FileInputStream		fis;
	DataInputStream		dis;
	Statement			stmt;
	PreparedStatement	pstmt;
	ResultSet			rs;
	DatabaseMetaData	dma;
	Vector				childThreads;
	String				url;
	static PreparedStatement globPrepStmnt;
	static Connection	globCon;
	String				sqlName;

	/**
	 * Each thread has its own set of SQL commands and its
	 * own statement objects. They share the same connection 
	 * though. They could equally well use different connections.
	 */
	SampleThread (String fileName) {
		try {

			// Open the sample sql file

			fis = new FileInputStream (fileName);
			dis = new DataInputStream (fis);
			childThreads = new Vector (10,10);
			sqlName=fileName;
		} catch (Exception e) {
			e.printStackTrace ();
		} // try-catch
	} // method SampleThread

	/**
	 * Waits for all child threads to complete before continuing.
	 */
	void waitForChildren () throws InterruptedException {
		// wait for child threads to finish
		for (int i=0; i<childThreads.size(); i++) {
			Thread curThread = (Thread)childThreads.elementAt(i);
			curThread.join();
		} // for
	} // method waitForChildren


	/**
	 * This routine does the work of this particular thread.
	 */
	public void run () {
		Thread thisThread = Thread.currentThread();
		thisThread.setName(sqlName);
		long startTime=0;
		long diffTime=0;

		// outer try block to handle IO errors etc
		try {
			while (dis.available()>0) {

				// read in lines until we get one terminated with a semi-colon

				boolean endFound=false;
				StringBuffer curStatement=new StringBuffer();
				while (!endFound) {
					String curLine = dis.readLine();
					if (curLine.length()==0) continue;
					if (curLine.charAt(0)==';') continue;
					curStatement.append(curLine);
					if (curStatement.charAt(curStatement.length()-1)==';') {
						endFound=true;
						// remove the trailing ;
						curStatement.setLength(curStatement.length()-1);
					} // if
				} // while

				// Got a full line, find out what it wants us to do

				char command = curStatement.charAt(0);
				curStatement.setCharAt(0,' ');
				String sql=curStatement.toString();
				sql=sql.trim();

				// Inner try block prevent SQL exceptions taking us out of the while loop
				try {
					startTime = System.currentTimeMillis();
					switch (command) {
					case 'c':
						// special command
						if (sql.equalsIgnoreCase("autocommit on")) {
							con.setAutoCommit (true);
							System.out.println ("Auto-commit is "+con.getAutoCommit());
						} else if (sql.equalsIgnoreCase("autocommit off")) {
							con.setAutoCommit (false);
							System.out.println ("Auto-commit is "+con.getAutoCommit());
						} else if (sql.equalsIgnoreCase("break")) {
							// Using a 'b' command in a script and setting a breakpoint
							// below, allows us to step through the next statement.
							System.out.println ("Breakpoint reached");
						} else if (sql.equalsIgnoreCase("use global prepared statement")) {
							// Use the global prepared statement set up by another thread.
							pstmt = globPrepStmnt;
							break;
						} else if (sql.equalsIgnoreCase("time")) {
							System.out.println ("Last command took: "+diffTime);
							break;
						} else if (sql.equalsIgnoreCase("set global prepared statement")) {
							// Save ine the static statement handle.
							// This allows other threads to access the statement.
							globPrepStmnt = pstmt ;
							break;
						} else if (sql.equalsIgnoreCase("use global connection")) {
							// Use the global prepared statement set up by another thread.
							con = globCon;
							stmt = con.createStatement ();
							break;
						} else if (sql.equalsIgnoreCase("set global connection")) {
							// Set the global connection so that other threads can use it
							globCon = con;
							break;
						} else if (sql.equalsIgnoreCase("set isolation READ_UNCOMMITTED")) {
							// Set the global connection so that other threads can use it
							con.setTransactionIsolation (Connection.TRANSACTION_READ_UNCOMMITTED);
							break;
						} else if (sql.equalsIgnoreCase("set isolation SERIALIZABLE")) {
							// Set the global connection so that other threads can use it
							con.setTransactionIsolation (Connection.TRANSACTION_SERIALIZABLE);
							break;
						} else if (sql.equalsIgnoreCase("close")) {
							// close the database
							if (stmt != null) stmt.close();
							stmt = null;
							con.close();
							con=null;
							break;
						} else if (sql.equalsIgnoreCase("rollback")) {
							con.rollback();
						} else if (sql.equalsIgnoreCase("commit")) {
							con.commit();
						} else if (sql.equalsIgnoreCase("show meta-data")) {
							// get metadata
							rs = dma.getTypeInfo();
							dispResultSet (rs);
							rs.close();
							rs = dma.getTables("","","%",null);
							dispResultSet (rs);
							rs.close();
						} else if (sql.equalsIgnoreCase("wait for children")) {
							// wait for child threads to complete
							waitForChildren();
						} else if (sql.equalsIgnoreCase("exit")) {
							// exit immediately
							System.exit(0);
						} else {
							throw new Exception ("Unknown command: "+sql);
						} // if-else
						break;
					case 'd':
						// load a driver
						Class.forName (sql);
						
						break;
					case 'e':
						// execute command with no results set
						stmt.executeUpdate (sql);
						break;
					case 'i':
						// get index info for table
						rs = dma.getIndexInfo (null,null,sql,false,false);
						dispResultSet (rs);
						rs.close();
						break;
					case 'o':
						// get url to open
						StringTokenizer tokens = new StringTokenizer (sql);
						String url = tokens.nextToken();

						// read in properties to pass to connection (optional)
						Properties props = new Properties();
						if (tokens.hasMoreTokens()) {
							String propsFile = tokens.nextToken();
							FileInputStream propsStream = new FileInputStream (propsFile);
							props.load (propsStream);
						} // if

						con = DriverManager.getConnection (url, props);

						// Get the DatabaseMetaData object and display
						// some information about the connection

						dma = con.getMetaData ();

						System.out.println("\nConnected to " + dma.getURL());
						System.out.println("Driver   " + dma.getDriverName());
						System.out.println("Version  " + dma.getDriverVersion());
						System.out.println("");

						// Create a Statement object so we can submit
						// SQL statements to the driver

						stmt = con.createStatement ();

						// Have a prepared statment reference handy

						pstmt = null;
						
						break;
					case 'p':
						// prepared statement
						pstmt = con.prepareStatement (sql);

						break;
					case 'q':
						// query with results set
						rs = stmt.executeQuery (sql);
						dispResultSet (rs);
						rs.close();
						break;
					case 's':
						// set parameters and execute
						StringTokenizer st = new StringTokenizer (sql, ",;", false);
						int i = 1;
						while (st.hasMoreTokens()) {
							String token=st.nextToken();
							if (token.equals(",")) continue;
							token=token.trim();
							pstmt.setString (i++, token);
						} // while
						pstmt.executeUpdate();
						break;
					case 't':
						// start off a thread with its own input file
						Thread newThread = new Thread (new SampleThread (sql));
						newThread.start();
						childThreads.addElement(newThread);
						break;
					default:
						throw new Exception ("Unknown command: "+command);
					} // switch
					diffTime = System.currentTimeMillis()-startTime;
				} catch (Exception e) {
					System.out.println ("Error processing statement: "+sql);
					e.printStackTrace ();
				} // try-catch
			} // while
		
			waitForChildren();

		} catch (Exception e) {
			e.printStackTrace ();
		} finally {
			try {
				dis.close();
				fis.close();
			} catch (Exception e) {
				e.printStackTrace ();
			} // try-catch
		} // try-catch

	} // method run


	//-------------------------------------------------------------------
	// dispResultSet
	// Displays all columns and rows in the given result set
	//-------------------------------------------------------------------

	private static void dispResultSet (ResultSet rs)
		throws SQLException
	{
		int i;

		ResultSetMetaData rsmd = rs.getMetaData ();
		int numCols = rsmd.getColumnCount ();

		for (i=1; i<=numCols; i++) {
			if (i > 1) System.out.print(",");
			System.out.print(rsmd.getColumnLabel(i));
		}
		System.out.println("");
		
		while (rs.next ()) {

			for (i=1; i<=numCols; i++) {
				if (i > 1) System.out.print(",");
				System.out.print(rs.getString(i));
			}
			System.out.println("");

		}
	}
} // class SampleThread 