
//Title:        HSA Parents' Corner
//Version:      
//Copyright:    Copyright (c) 1998
//Author:       Ozgur Balsoy
//Company:
//Description:  Online Parental Access to the School Database


package hsa.util;

import hsa.db.dbHashtable;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.*;
import javax.servlet.ServletException;

/**
 * Servlets that list the servlets in this virtual host
 * and their init parameters.
 *
 * @author Francis J. Lacoste
 */
public class Parser {

  /**
   * Retrieves the HTML file and replaces the variables with their values.
   * This version of the method does not take any record parser. Record
   * parsers handle &lt;record&gt; tags in the HTML templates.
   *
   * @exception  ServletException if there is any sort of Exception
   *
   */
  public static void parseHtml
  (
   String templateFile, 
   dbHashtable fields,
   PrintWriter out
    )
       throws IOException, ServletException, SQLException
  {
    parseHtml(templateFile, fields, out, null);
  }
    

  /**
   * Retrieves the HTML file and replaces the variables with their values.
   * This version of the method <b>requires</b> a record parser. Record
   * parsers handle &lt;record&gt; tags in the HTML templates.
   *
   * @exception  ServletException if there is any sort of Exception
   *
   */
  public static void parseHtml (
				String templateFile, 
				dbHashtable fields,
				PrintWriter out,
				RecordParsable rparser
				 )       throws IOException, ServletException, SQLException
  {
    //response.setContentType("text/html");
    //PrintWriter out = response.getOutputStream();
    //String filename = request.getPathTranslated();

    File htmlfile = new File( templateFile );

    if(!htmlfile.isFile()) 
      {
	throw new IOException("Could not open the file "+ templateFile +",");
      } else {
	
	BufferedReader lines = 
	  new BufferedReader
	  (new InputStreamReader
	   (new FileInputStream( htmlfile )));

	String aline, record="";
	int pos;
	boolean recordCollect = false;
	while ( ( aline = lines.readLine() ) != null )
	  {
	    if(!recordCollect)
	      {
		pos = aline.indexOf("<record>");
		if(pos >= 0 ) // this line needs a special treatment b/of the <record> tag.
		  {
		    if(pos>0) // there must be leading characters, so let parseLine handle those.
		      parseLine(fields, aline.substring(0, pos), out);
		    
		    aline = aline.substring(pos+8); // trim the leading chars, as well as <record> .
		    recordCollect = true;
		  }
		else // no record tag, just continue.
		  Parser.parseLine(fields, aline, out);
	      }

	    if(recordCollect)
	      {
		pos = aline.indexOf("</record>");
		if(pos >= 0) // <record> tag is closed.
		  {
		    record += aline.substring(0, pos); // code between tags are captured.
		    
		    // put the record treatment here!!!
		    if(rparser != null) {// caller method may not provide a record parser! 
		      while( rparser.handleRecord(fields) ) {
			      parseLine(fields, record, out); 
		      }
		    } else 
		      // put the record treatment here!!!
		      aline = aline.substring(pos+9);   // trim the closing record tag.
		    Parser.parseLine(fields, aline, out); // print the rest of the line.
		    recordCollect = false;
		  }
		else
		  record += "\n"+aline; // tag is not closed and start collecting
	      }
	  }
    lines.close();
    }
  }
  
  public static void parseLine
  (dbHashtable fields,
   String aline,
   PrintWriter out
    )
       throws IOException, ServletException
  {
    String value="";
    int pos = aline.indexOf("$"); // each variable must start with '$'.
    while (pos >= 0 && aline.length()>1)
    {
	    if( pos != 0 )
	    {
	      if( aline.charAt(pos-1) == '\\' )       // escape character found!
	        aline = aline.substring(0, pos-1) + aline.substring(pos);
        out.print( aline.substring(0, pos) );   // replace esc+$ w/ $
	                                              // and print the heading part.
	      aline = aline.substring(pos);           // continue w/ the rest.
	    }
	    // first char after $ must be an identifier starter usu. alphabetic chars.
	    // we leave the job to Java.
	    else if( Character.isUnicodeIdentifierStart(aline.charAt(1)) )
      {
	      aline = Parser.parseVar( fields, aline, out  );
	    }
      else {
	      // if it is not a starter, then print $ and continue w/ the rest.
	      out.print( aline.substring(0, 1) );
	      aline = aline.substring(1);
	    }
	    pos = aline.indexOf("$");
    }
    out.println( aline );
  }

  public static String parseVar(dbHashtable fields, String aline, PrintWriter out)
        throws IOException, ServletException
  {
    //String E="empty", F="full";
    //boolean zerotest=false;

    Log.debug("(parsevar) aline: '"+aline+"'");
    if(aline.length() > 1)
    {
	    int pos=1;
	    String word="";
	    // before coming here, we made sure that there is at least one char
	    // which can make us an identifier.
	    //boolean status=false;
	    while( aline.length()>pos && Character.isUnicodeIdentifierPart( aline.charAt(pos) ) )
	    {
        word += aline.charAt(pos);
	      pos++;
	    }
      if( aline.length()>pos )
        aline = aline.substring(pos);
      else
        aline = "";
    	if( !word.equals("") ) {
        Object value = fields.get(word);
        if(value != null)
          aline = value.toString() + aline;
      }
      Log.debug("(parsevar) word: '"+word+"' aline: '"+aline+"' pos: "+pos );
    }
    return aline;
  }

  /*
	if (aline.charAt(pos)=='#')  {
	  status=true;  //default E F
	  pos++;
	  if (aline.charAt(pos)=='0') {
	    pos++;
	    zerotest=true;
	  }
	}
	else if (aline.charAt(pos)=='*') {
	  status=true;  E=""; F="*";
	  pos++;
	}
	else if (aline.charAt(pos)=='%') {
	  word=(String)fields.get("start")+word;
	  status=true;
	  pos++;
	  if (aline.charAt(pos)=='#')
	    pos++;
	  else  {
	    E=""; F=(String)fields.get( word);
	  }
	}
	if (aline.charAt(pos)=='`') {
	  status=true;
	  pos++;
	  if (aline.charAt(pos)=='x') {
	    String n="";  pos++;
	    while ( aline.charAt(pos)!='`') {
	      n+=aline.charAt(pos);
	      pos++;
	    }
	    pos++;
	    String c=(String)fields.get( word);
 	    if (c!=null)
	      F=Integer.toString(Integer.parseInt(n)*Integer.parseInt(c));
	    else E="0";

	  }
	}
	if( !word.equals("") )
	  {
	    String s=(String)fields.get(word);
	    if (status){
	      if ( s==null || s.equals("") )
		out.print(E); //prints value
	      else if (zerotest && s.equals("0") ) out.print(E);
	      else out.print(F);
	    }
	    else {
	      if ( s==null ) s="null";
	      out.print(s);
	      }
	    aline = aline.substring(pos);
	  }
      }

    return aline;
  } */
  ///////////////////////////////==============
  public static BufferedReader parseHtmlOpen (String templateFile)
       throws IOException, ServletException, SQLException
  {
   File htmlfile = new File( templateFile );

    if(!htmlfile.isFile()) 
      {
	throw new IOException("Could not open the file "+ templateFile +",");
      } else {
	
	 BufferedReader lines = 
	  new BufferedReader
	  (new InputStreamReader
	   (new FileInputStream( htmlfile )));
    return lines;
     }
 
  }
  public static String parseHtmlUntilRecord (
				dbHashtable fields,
				PrintWriter out,
				BufferedReader lines
				 )       throws IOException, ServletException, SQLException
  {
	String aline, record="";
	int pos;
	boolean recordCollect = false;
	while ( ( aline = lines.readLine() ) != null )  {
	  if(!recordCollect)	    {
	    pos = aline.indexOf("<record>");
	    if(pos >= 0 ) { // this line needs a special treatment b/of the <record> tag.
	      if(pos>0) // there must be leading characters, so let parseLine handle those.
		parseLine(fields, aline.substring(0, pos), out);
	      aline = aline.substring(pos+8); // trim the leading chars, as well as <record> .
	      recordCollect = true;
	    } else // no record tag, just continue.
	      Parser.parseLine(fields, aline, out);
	  }
	  else if(recordCollect)   {
	    pos = aline.indexOf("</record>");  //</record should be at the end
	    if(pos >= 0)  {// <record> tag is closed.
	      record += aline.substring(0, pos); // code between tags are captured.
	      return record;
	    } else
	      record += "\n"+aline; // tag is not closed and start collecting
	  }
	}
    return "";
  }

   ///////////////////////////////==============

  public static void parseHtmlAfterRecord (
				
				dbHashtable fields,
				PrintWriter out,
				BufferedReader lines
				  )       throws IOException, ServletException, SQLException
  {

    String aline="";
    while ( ( aline = lines.readLine() ) != null )
      {
	
	Parser.parseLine(fields, aline, out);
	
      }
  }

}