import java.io.*;
import java.net.*;
import java.util.*;

import injektilo.net.*;

public class Pop3Test {
  public static void main(String args[])
      throws UnknownHostException, IOException {
    if (args.length < 3) {
      System.out.println("Usage: java Pop3Test host user pass");
      System.exit(1);
    }
        
    System.out.println("Pop3Test - type HELP for commands");
      
    Pop3Connection pop3 = new Pop3Connection();

    pop3.setDebugOut(System.out);
    pop3.setDebug(true);
    
    pop3.setAuthentication(Pop3Connection.USER);
    
    String host, user, pass;
    
    int i = 0;
    if (args[i].equals("-apop")) {
      pop3.setAuthentication(Pop3Connection.APOP);
      ++i;
    }    
    host = args[i++];
    user = args[i++];
    pass = args[i++];
    
    pop3.open(host);
    if (!pop3.authenticate(user, pass)) {
      pop3.quit();
      pop3.close();
      System.exit(1);
    }
    
    BufferedReader in = new BufferedReader(
      new InputStreamReader(System.in), 1);
    StringTokenizer st;
    String s;
    
    while (true) {
      System.out.print("> ");
      st = new StringTokenizer(in.readLine());
      if (st.countTokens() == 0) {
        continue;
      }
      try {
        s = st.nextToken();
        s = s.toUpperCase();
        if (s.startsWith("S")) { // STATUS
          pop3.status();
          System.out.println("totalMessages = " + pop3.getTotalMessages()
            + " totalOctets = " + pop3.getTotalOctets());
        } else if (s.startsWith("L")) { // LIST
          if (st.hasMoreTokens()) {
            pop3.list(Integer.parseInt(st.nextToken()));            
            System.out.println("octets = " 
              + pop3.getOctets());
          } else {
            pop3.list();
            System.out.println(pop3.getOctetsVector());
          }
        } else if (s.startsWith("RET")) { // RETRIEVE
          pop3.retrieve(Integer.parseInt(st.nextToken()));
          BufferedReader br = new BufferedReader(new InputStreamReader(pop3.getInputStream()));
          while ((s = br.readLine()) != null) {
            System.out.println(s);
          }
        } else if (s.startsWith("D")) { // DELETE
          pop3.delete(Integer.parseInt(st.nextToken()));
        } else if (s.startsWith("N")) { // NOOP
          pop3.noOp();
        } else if (s.startsWith("RES")) { // RESET
          pop3.reset();
        } else if (s.startsWith("T")) { // TOP
          pop3.top(Integer.parseInt(st.nextToken()),
            Integer.parseInt(st.nextToken()));
          pop3.readTo(System.out);
        } else if (s.startsWith("U")) { // UIDL
          if (st.hasMoreTokens()) {
            pop3.uidl(Integer.parseInt(st.nextToken()));
            System.out.println("uid = "
              + pop3.getUid());
          } else {
            pop3.uidl();
            System.out.println(pop3.getUidVector());
          }
        } else if (s.startsWith("Q")) { // QUIT
          pop3.quit();
          break;
        } else if (s.startsWith("H")) { // HELP
          System.out.println(help);
        }
      } catch (NoSuchElementException e) {
        System.out.println(help);
      }
    }
    
    pop3.close();
  }
  
  private static String help = 
      "  STATUS\n"
    + "  LIST [message]\n"
    + "  RETRIEVE message\n"
    + "  DELETE message\n"
    + "  NOOP\n"
    + "  RESET\n"
    + "  TOP message lines\n"
    + "  UIDL [message]\n"
    + "  QUIT";
}