/**
 * Pop3Inbox.java
 *
 * @author Jason Diamond
 * @version 1.1.2
 *
 * <p>Copyright (c) 1998 Jason Diamond</p>
 *
 * <p>sendangels@usa.net<br>
 * http://www.geocities.com/ResearchTriangle/Thinktank/8343/</p>
 *
 * History:
 * 
 * 10-Nov-1998 v1.1.1 released
 * 15-Nov-1998 v1.1.2
 * <ul>
 * <li>only sends the STAT command after the first call to getMessageCount
 * <li>messages[] cache is created after the first call to getMessage
 * <li>getMessage throws IndexOutOfBoundsException if msgnum is not valid
 * <li>expunge checks to see if a message in the cache is null before calling renumber
 * </ul>
 * 
 */

package injektilo.mail;

import java.io.*;
import java.util.*;
import javax.mail.*;
import javax.mail.event.*;

import injektilo.net.*;

public class Pop3Inbox
    extends Pop3Folder
{
  private int count = -1;
  private Pop3Message[] messages;
  int[] msgnums;
  
  public Pop3Inbox(Pop3Store store) {
    super(store, "INBOX");
    this.store = store;
  }
  
  public Folder getParent()
      throws MessagingException
  {
    return new Pop3DefaultFolder(store);
  }
  
  public boolean exists()
      throws MessagingException
  {
    return true;
  }
  
  public int getType()
      throws MessagingException
  {
    return HOLDS_MESSAGES;
  }
  
  public boolean hasNewMessages()
      throws MessagingException
  {
    return (getMessageCount() > 0);
  }
  
  public void open(int mode)
      throws MessagingException
  {
    notifyConnectionListeners(ConnectionEvent.OPENED);
  }
  
  public void close(boolean expunge)
      throws MessagingException
  {
    notifyConnectionListeners(ConnectionEvent.CLOSED);
  }
  
  public boolean isOpen() {
    return true;
  }
  
  public Flags getPermanentFlags() {
    return new Flags();
  }
  
  public int getMessageCount()
      throws MessagingException
  {
    if (count == -1) {
      try {
        if (store.pop3.status()) {
          count = store.pop3.getTotalMessages();
        } else {
          throw new MessagingException("getMessageCount failed");
        }
      } catch (IOException e) {
        throw new MessagingException("getMessageCount failed", e);
      }
    }

    return count;
  }

  public Message getMessage(int msgnum)
      throws MessagingException
  {
    if (messages == null) {
      if (count == -1) {
        getMessageCount();
      }
      messages = new Pop3Message[count + 1];
      for (int i = 1; i <= count; ++i) {
        messages[i] = null;
      }
      msgnums = new int[count + 1];
      for (int i = 1; i <= count; ++i) {
        msgnums[i] = i;
      }
    }
    
    if ((msgnum > 0) & (msgnum <= count)) {
      Pop3Message message = messages[msgnums[msgnum]];
      if (message == null) {
        message = new Pop3Message(this, msgnum, msgnums[msgnum]);
        messages[msgnums[msgnum]] = message;
      }
      return message;
    } else {
      throw new IndexOutOfBoundsException(count + " messages in folder");
    }
  }

  public Message[] expunge()
      throws MessagingException
  {
    int i = 1;
    Vector v = new Vector();
    
    while (i <= count) {
      Message m = messages[msgnums[i++]];
      if (m != null) {
        Flags f = m.getFlags();
        if (f.contains(Flags.Flag.DELETED)) {
          v.addElement(m);
        }
      }
    }
    
    Pop3Message[] a = new Pop3Message[v.size()];
    v.copyInto(a);
    
    try {
      for (i = 0; i < a.length; ++i) {
        a[i].expunge();
        removeMessageNumber(a[i].getMessageNumber());
      }
    } catch (IOException ex) {
      throw new MessagingException("expunge failed", ex);
    }
    
    for (i = 1; i <= count; ++i) {
      if (messages[msgnums[i]] != null) {
        messages[msgnums[i]].renumber(i);
      }
    }
    
    notifyMessageRemovedListeners(true, a);
    
    return a;
  }
  
  private void removeMessageNumber(int i) {
    while (i < count) {
      msgnums[i] = msgnums[++i];
    }    
    --count;
  }
}