/**
 * Pop3Message.java
 *
 * @author Jason Diamond
 * @version 1.1.2
 *
 * <p>Copyright (c) 1998 Jason Diamond</p>
 *
 * <p>sendangels@usa.net<br>
 * http://www.geocities.com/ResearchTriangle/Thinktank/8343/</p>
 *
 * <ul><li>v1.1.1 released
 * <li>v1.1.2<ul>
 * <li>added pop3num to keep track of message number on server after renumbering
 * </ul><li>v1.1.3<ul>
 * <li>gets UIDL from server and adds X-UIDL header to message when retrieving
 * </ul>
 *
 */

package injektilo.mail;

import java.io.*;
import javax.mail.*;
import javax.mail.internet.*;

public class Pop3Message
    extends MimeMessage
{
  Pop3Folder folder;
  int pop3num;
  
  public Pop3Message(Pop3Folder folder, int msgnum, int pop3num) 
      throws MessagingException
  {
    super(folder, msgnum);
    this.folder = folder;
    this.pop3num = pop3num;
    try {
      if (folder.store.pop3.uidl(pop3num)) {
      }
      folder.store.pop3.retrieve(pop3num);
      InputStream is = folder.store.pop3.getInputStream();
      headers = new InternetHeaders(is);
      ByteArrayOutputStream os = new ByteArrayOutputStream();
      int len;
      byte[] buf = new byte[1024];
      while ((len = is.read(buf, 0, 1024)) != -1) {
        os.write(buf, 0, len);
      }
      content = os.toByteArray();
    } catch (IOException e) {
      throw new MessagingException("getMessage failed", e);
    }    
  }
  
  void expunge() 
      throws IOException
  {
    folder.store.pop3.delete(pop3num);
    setExpunged(true);
  }
  
  void renumber(int newnum) {
    msgnum = newnum;
  }
  
}