CREATE OR REPLACE PACKAGE element_pkg AS

TYPE element_rec_type IS RECORD
        (Name           VARCHAR2(15),
        Symbol          VARCHAR2(3));

TYPE elements_rec_type IS RECORD
	(Name		VARCHAR2(15),
      	Symbol 		VARCHAR2(3),
	Atomic_Number	NUMBER(3),
	Mass		NUMBER(10),
	Melting_Point	VARCHAR2(10),
	Boiling_Point	VARCHAR2(10),
	Number_of_PEs	NUMBER(3),
	Number_of_Ns	NUMBER(3),
	Classification	VARCHAR2(21),
	Crystal_Struct	VARCHAR2(12),
	Shells		VARCHAR2(30),
	Date_of_Disc	VARCHAR2(15),
	Discoverer	VARCHAR2(45),
	Name_Symbol_Org	VARCHAR2(60),
	Uses		VARCHAR2(75),
	Sources		VARCHAR2(75),
	Density		VARCHAR2(10),
	HeatVapor	VARCHAR2(10),
	HeatFusion	VARCHAR2(10),
	Electrical_Cond	VARCHAR2(10),
	Thermal_Cond	VARCHAR2(10),
	Specific_Heat	VARCHAR2(10),
	Group_Number	VARCHAR2(2),
	Standart_State	VARCHAR2(10),
	Colour		VARCHAR2(35),
	Meaning_of_Name	VARCHAR2(100));

TYPE URL_rec_type IS RECORD
	(Name		NUMBER(15),
	URL		VARCHAR2(100),
	Link_Info	VARCHAR2(100));

    PROCEDURE get_element_by_name(anelement IN VARCHAR2);

    PROCEDURE display_rec(aRec IN element_rec_type);

    PROCEDURE display_error_msg;
END element_pkg;
/
CREATE OR REPLACE PACKAGE BODY element_pkg AS

    PROCEDURE get_element_by_name(anelement IN VARCHAR2)
    IS
        CURSOR element_cur(anelement IN VARCHAR2)
        IS
            SELECT Name, Symbol
            FROM elements_table
            WHERE (LOWER(Name) = LOWER(anelement));

        aRec   element_rec_type;
        item_found NUMBER(5) := 0;

    BEGIN

        IF NOT element_cur%ISOPEN THEN          -- open the cursor if
            OPEN element_cur(anelement);             -- not open yet
        END IF;

        -- Fetch rows into PL/SQL table
        LOOP
            FETCH element_cur INTO aRec;
            IF element_cur%NOTFOUND THEN
		display_error_msg;
		-- EXIT;
	    ELSE
		display_rec(aRec);
	    END IF;
        END LOOP;

        CLOSE element_cur;

    END get_element_by_name;

    PROCEDURE display_rec(aRec IN element_rec_type)
	IS
		background VARCHAR2(120) := 'http://www.ege.edu.tr/~balsoy/' ||
 			'background/tan_paper.gif';
	BEGIN
htp.title('Selamlar');
--       		htp.title(aRec.Name);
		htp.p('<body background=' || background || '>');
--		htp.header(1,aRec.Name);
--		htp.line;
--		htp.bold('Symbol:');
--		htp.p(aRec.Symbol);
--		htp.p('</body>');

	END display_rec;

    PROCEDURE display_error_msg
    IS
		background VARCHAR2(120) := 'http://www.ege.edu.tr/~balsoy/' ||
 			'background/tan_paper.gif';
    BEGIN
        htp.title('Query Results');
        htp.p('<body background=' || background || '>');
	htp.p('<hr width=50%>');
        htp.header(1,'<center>No element is found in this name.</center>');
	htp.p('<hr width=50%>');

    END display_error_msg;

END element_pkg;
/
