// ::::::::::::::
// DrawCanvas.java
// ::::::::::::::

// Developed by Mehmet Sen. Spring 1996
// Ported to Java 1.1 by Ozgur Balsoy. Fall 1997
// Networking capabilities added by Ozgur Balsoy

import java.awt.*; 
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class DrawCanvas extends Canvas {

  Paint parent;
  DrawCanvas mySelf = this;
  String curMode="Rect";
  mPoint curObj;
  public boolean fill=true;
  mSlide objList=null;
  Vector slideList=new Vector(10, 5);
  
  Hashtable colors;
  String    currentColor = "red";
  String    fillColor = "black";
  String    backgroundColor="blue";
  Font      currentFont;
  boolean   interlace=false;
  
  mText     lasttext=null;
  String    script, section, session, title, teacher;
  URL       baseUrl;
  
  public DrawCanvas(Paint _parent) {
    
    parent = _parent;
    setBackground(Color.blue);
    setForeground(Color.yellow);
    
    baseUrl= parent.getDocumentBase();
    script = parent.getParameter("javaTransferScript");
    section= parent.getParameter("scriptSection");
    session= parent.getParameter("lectureCode");
    title  = parent.getParameter("lectureTitle");
    teacher= parent.getParameter("lectureInstructor");
    
    colors=new Hashtable();
    colors.put("black", 	Color.black);
    colors.put("blue",  	Color.blue);
    colors.put("cyan", 	Color.cyan);
    colors.put("darkGray", 	Color.darkGray);
    colors.put("gray", 	Color.gray);
    colors.put("green", 	Color.green);
    colors.put("lightGray",	Color.lightGray);
    colors.put("magenta", 	Color.magenta);
    colors.put("orange", 	Color.orange);
    colors.put("pink", 	Color.pink);
    colors.put("red", 	Color.red);
    colors.put("white", 	Color.white);
    colors.put("yellow", 	Color.yellow);
    
    currentFont = new Font("Helvetica", Font.PLAIN, 12);
    slideList.addElement(objList=
			 new mSlide("Slide1", getSize().width, getSize().height, "blue"));
    
    // Notice the construction of two listener objects, ourMouseListener
    // and ourMotionListener. Both are constructed from the class 
    // definitions, called anonymous classes.
    
    // "You should consider using an anonymous class instead of
    // [a top level class or] a local class if:
    // . The class has a very short body.
    // . Only one instance of the class is needed.
    // . The class is used right after it is defined.
    // . The name of the class does not make your code any easier to understand."
    // [Java in a Nutshell, pg. 119]
    
    // We define our version of mouse adapters here 
    // because we will not use all of the MouseListener methods.
    
    // MouseListener is an interface and all of its methods are defined abstract.
    // This forces us to overwrite all the methods even though we do not need some.
    // To overcome this problem, adapter classes are developed. These are 
    // classes implementing interfaces with empty method bodies. Hence, we only
    // overwrite those methods that we need. 
    
    // In this case, we need mousePressed and mouseReleased.
    MouseListener ourMouseListener = new MouseAdapter() {
      // from MouseAdapter
      public void mousePressed(MouseEvent e) {
	
	int x=e.getX();
	int y=e.getY();
	if (curMode.equals("Rect"))
	  curObj=new mRectangle(mySelf, x, y, x, y, currentColor, fillColor);
	if (curMode.equals("Round"))
	  curObj=new mRound(mySelf, x, y, x, y, currentColor, fillColor);
	else if (curMode.equals("Circ"))
	  curObj=new mCircle(mySelf, x, y, x, y, currentColor, fillColor);
	else if (curMode.equals("Line"))
	  curObj=new mLine(mySelf, x, y, x+1, y+1, currentColor, fillColor);
	else if (curMode.equals("Text"));
	
      }
      
      // from MouseAdapter
      public void mouseReleased(MouseEvent e) {
	
	int x=e.getX();
	int y=e.getY();
	if (curMode.equals("Text")) {
	  if (lasttext == null) {
	    lasttext=new mText(mySelf, x, y, currentColor);
	    showFontDialog();
	  }
	} else {
	  Graphics g = getGraphics();
	  g.setPaintMode();
	  curObj.setEnds(x, y);
	  curObj.setFill(fill);
	  curObj.draw(g);
	  objList.addElement(curObj);
	  g.dispose();
	}
      }
    };
    
    // We also need mouseDragged from MouseMotionListener.
    MouseMotionListener ourMotionListener = new MouseMotionAdapter() {
      public void mouseDragged(MouseEvent e) {
    
	int x=e.getX();
	int y=e.getY();
	if (!curMode.equals("Text")) {
	  Graphics g = getGraphics();
	  g.setXORMode(Color.black);
	  curObj.draw(g);
	  curObj.setEnds(x, y);
	  curObj.draw(g);
	  g.dispose();
	}
	
      }
    };
    
    addMouseListener( ourMouseListener );
    addMouseMotionListener( ourMotionListener );
  } 
  
  public void setDrawMode(String s) {
    if(s.equals("Clear")) {
      objList.removeAllElements();
      repaint();
    } else if(s.equals("Redraw")) repaint();
    else curMode=s;
  }
  
  public void paint(Graphics g) {
    
    if (objList != null) {
      g.setColor(getHashedColor(objList.bgColor));
      g.fillRect(0, 0, getSize().width, getSize().height);
      
      for(Enumeration e=objList.elements(); e.hasMoreElements() ;) 
	((mPoint)e.nextElement()).draw(g);
      
    }
  }
  
  String constructHeader() {
    
    String key;
    Color tempColor;
    
    String result = "Session " + session + "\n";
    result += "Title " + title + "\nInstructor " + teacher + "\n";
    for (Enumeration e = colors.keys() ; e.hasMoreElements() ;) {
      tempColor=(Color)colors.get(key=(String)e.nextElement());
      result += "Color " + key + " " + 
	Integer.toString(tempColor.getRed()) + " " +
	Integer.toString(tempColor.getGreen()) + " " +
	Integer.toString(tempColor.getBlue()) + "\n";
    }
    
    if (slideList != null) 
      for(Enumeration e=slideList.elements(); e.hasMoreElements() ;) {
	result += ((mSlide)e.nextElement()).makeString();
      }
    
    return result;
  }
  
  
  public void sendData() {
    
    String strReturn = "", strData = "";  
    Socket sckS = null;
    try {
      sckS = new Socket(baseUrl.getHost(), baseUrl.getPort());
      
      DataOutputStream os
	= new DataOutputStream(sckS.getOutputStream());
      BufferedReader is = new BufferedReader(new InputStreamReader(sckS.getInputStream()));
      //DataInputStream is = new DataInputStream(sckS.getInputStream());
      
      strData = "section=" + section + "&code=" + session + "&information=";
      strData += constructHeader();
      os.writeBytes("POST " + script
		    + " HTTP/1.0\r\n"
		    + "Content-type: application/octet-stream\r\n"
		    + "Content-length: "
                    + strData.length() + "\r\n\r\n");
      os.writeBytes(strData);
      
      String line="";
      // get rid of the header.
      while (((line = is.readLine()) != null)
	     && !(line.startsWith("Content-type:")));
      
      // save the information
      while ((line = is.readLine()) != null)
	strReturn += line + "\n";
      
      is.close();
      os.close();
      
    } 
    catch (Exception e) {
      
      if (sckS != null) {
	try { sckS.close(); }   
	catch (IOException ex) {}
	
      }
    }
    System.out.println("output:\n"+strData);
    
    System.out.println("input:\n"+strReturn);
    
  }
  
  public Color getHashedColor(String keyColor) {
    return (Color)colors.get(keyColor);
  }
  
  public void showColorDialog() {
    Hashtable cu=new Hashtable();
    cu.put("draw color", currentColor);
    cu.put("fill color", fillColor);
    cu.put("background", backgroundColor);
    
    ColorFrame f=new ColorFrame(this, colors, cu);
    f.setSize(700, 200);
    f.show();
  }
  
  public void getNewColors(Hashtable c, Hashtable cu) {
    
    colors=c;
    currentColor = (String)cu.get("draw color");
    fillColor = (String)cu.get("fill color");
    backgroundColor = (String)cu.get("background");
    objList.changeBG(backgroundColor);
    repaint();
  }
  
  public void showFontDialog() {
    
    FontDialog f=new FontDialog(this, currentFont);
    f.setSize(500,300);
    f.show();
  }
  
  public void getNewFont(Font f, String text, boolean change) {
    
    currentFont = f;
    if (lasttext != null && change) {
      Graphics g = getGraphics();
      g.setPaintMode();
      lasttext.setData(text, f);
      lasttext.draw(g);
      objList.addElement(lasttext);
      g.dispose();
      lasttext=null;
      repaint();
    }
    if (!change) lasttext=null;
  }
  
  public void newSlide() {
    
    if (objList != null) {
      String no=String.valueOf(slideList.size()+1);
      objList=new mSlide("Slide"+no, getSize().width, getSize().height, backgroundColor);
      slideList.addElement(objList);
    }
    repaint();
  }
  
  public void showPrevSlide() {
    
    if (objList != null) {
      int slideno=slideList.indexOf(objList);
      if (slideno > 0) 
	objList = (mSlide) slideList.elementAt(slideno-1);
      else objList = (mSlide) slideList.lastElement();
    }
    repaint();
  }
  
  public void showNextSlide() {
    
    if (objList != null) {
      int slideno=slideList.indexOf(objList);
      if (slideno < slideList.size()-1) 
	objList = (mSlide) slideList.elementAt(slideno+1);
      else objList = (mSlide) slideList.firstElement();
    }
    repaint();
  }
}
