/*
 * Gary Cornell and Cay S. Horstmann, Core Java (Book/CD-ROM)
 * Published By SunSoft Press/Prentice-Hall
 * Copyright (C) 1996 Sun Microsystems Inc.
 * All Rights Reserved. ISBN 0-13-565755-5
 *
 * Permission to use, copy, modify, and distribute this 
 * software and its documentation for NON-COMMERCIAL purposes
 * and without fee is hereby granted provided that this 
 * copyright notice appears in all copies. 
 * 
 * THE AUTHORS AND PUBLISHER MAKE NO REPRESENTATIONS OR 
 * WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHORS
 * AND PUBLISHER SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED 
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING 
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
 
/**
 * @version 1.00 07 Feb 1996 
 * @author Cay Horstmann
 */

/*
 * Modified by Ozgur Balsoy for the Term Project of CSE791.
 *
 */

import java.awt.*;
import java.awt.event.*;

public class FontDialog extends Frame
                     implements ActionListener, ItemListener
{  private void add(Component c, GridBagLayout gbl,
		    GridBagConstraints gbc,
		    int x, int y, int w, int h)
  {  
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = w;
    gbc.gridheight = h;
    gbl.setConstraints(c, gbc);
    add(c);
  }
  
  public FontDialog(DrawCanvas _parent, Font currentFont)
  {  
    parent = _parent;  
    setTitle("FontDialog");
    GridBagLayout gbl = new GridBagLayout();
    setLayout(gbl);
    
    style = new List(4, false);
    String[] flist= getToolkit().getFontList();
    for(int i=0; i<flist.length; i++) {
      style.addItem(flist[i]);
      if (flist[i].equals(currentFont.getName())) style.select(i);
    }
    
    bold = new Checkbox("Bold");
    bold.setState(currentFont.isBold());
    
    italic = new Checkbox("Italic");
    italic.setState(currentFont.isItalic());
    
    Label label = new Label("Size: ");
    test = new TextField("Type your text here and hit Enter!");
    size = new TextField(String.valueOf(currentFont.getSize()));
    sample = new Canvas();
    
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 20;
    gbc.weighty = 0;
    add(style, gbl, gbc, 0, 0, 1, 3);
    gbc.weightx = 100;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.CENTER;
    add(bold, gbl, gbc, 1, 0, 1, 1);
    add(italic, gbl, gbc, 2, 0, 1, 1);
    add(label, gbl, gbc, 1, 1, 1, 1);
    gbc.fill = GridBagConstraints.HORIZONTAL;
    add(size, gbl, gbc, 2, 1, 1, 1);
    Button b;
    add(b=new Button("Update"), gbl, gbc, 1, 2, 1, 1);
    b.addActionListener(this);
    add(b=new Button("Quit"), gbl, gbc, 2, 2, 1, 1);
    b.addActionListener(this);
    gbc.weighty = 0;
    add(test, gbl, gbc, 0, 3, 4, 1);     
    
    gbc.anchor = GridBagConstraints.SOUTH;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.weighty = 100;
    add(sample, gbl, gbc, 0, 4, 4, 2);
    
  }
  
  //public boolean handleEvent(Event evt)
  //{  if (evt.id == Event.WINDOW_DESTROY) System.exit(0);
  //   return super.handleEvent(evt);
  //}

  public void reDraw() {
    int m = (bold.getState() ? Font.BOLD : 0) +
      (italic.getState() ? Font.ITALIC : 0) ; 
    Font f = new Font(style.getSelectedItem(), m, Integer.parseInt(size.getText()));
    Graphics g=sample.getGraphics();
    g.setColor(Color.lightGray);
    g.fillRect(0, 0, sample.getSize().width, sample.getSize().height);
    g.setColor(Color.black);
    g.setFont(f);
    g.drawString(test.getText(), 0, sample.getSize().height/2);
    g.dispose();
  }    
  
  public void updateFontInfo(boolean update) {
    int m = (bold.getState() ? Font.BOLD : 0) +
      (italic.getState() ? Font.ITALIC : 0) ; 
    Font f = new Font(style.getSelectedItem(), m, Integer.parseInt(size.getText()));
    parent.getNewFont(f, test.getText(), update);
  }
  
  public void itemStateChanged(ItemEvent e) {

    reDraw();
  }

  public void actionPerformed(ActionEvent e) {
    String s = e.getActionCommand();
    
    if (s.equals("Quit")) {
      updateFontInfo(false);
      dispose();
    } else if (s.equals("Update")) {
      updateFontInfo(true);
      dispose();
    }
    reDraw();
  }

//   public static void main(String[] args)
//   {  Frame f = new FontDialog();
//      f.resize(400, 200);
//      f.show();  
//   }
   
   private DrawCanvas parent;
   private List style;
   private Checkbox bold;
   private Checkbox italic;
   private TextField size;
   private TextField test;
   private Canvas sample;

}

