// import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.awt.TextArea;
// Grabbed from Doug Aberdeen who grabbed it from the Alpha-3 release
import Document;
import Tag;
import TagRef;
import Parser;


public class Snarf {

  static Hashtable visitedURLs = new Hashtable();
  protected Vector directRefs; // List of tagRefs
  protected URL root;
  protected String name;
  protected String snarfDir;
  protected TextArea logWindow;
  protected boolean ImagesToo;
  protected boolean OnlyHTML;

  static private int SnarfCount = 0;
  static private final int maxSnarfCount = 1000;
  static public int ResetVisitedURLs() {
    int count = visitedURLs.size();
    visitedURLs = new Hashtable();
    return count;
  }
  private String NoLabel(String s) {
      if (s == null) return null;

      // If this ends in a label, strip out the label
      int LabelPos = s.lastIndexOf('#');
      // Note: We assume that there is no other # in the
      // URL, is that in fact accurate?
      if (LabelPos >= 0) {
              s = s.substring(0, LabelPos);
              // System.out.println("Label at " + LabelPos +
              //    " yields " + s);
      }
      return s;
  }
  private boolean ShouldSnarf(TagRef t) {
     boolean snarfable = t.tag.name.equals("a") && !t.isEnd;
     if (snarfable) {
        // make sure there is an href to an HTML file
        String href = t.getParameter("href");
        snarfable = (href != null);
        href = NoLabel(href);
        if (snarfable && OnlyHTML) {
             snarfable = (href.endsWith(".html") || href.endsWith(".htm"));
       }
     }
     // One last chance, see if its an image and 
     // see if we are snarfing images too ...
     if (!snarfable && ImagesToo) {
        snarfable = t.tag.name.equals("img") && !t.isEnd;

        String src = t.getParameter("src");
        snarfable = (src != null);
     }
     return snarfable;
  }

  private boolean IsDirectDescendant(URL parent, URL child) {
      String parent_name = parent.toExternalForm();
      String child_name = child.toExternalForm();

      if (parent_name.equals(child_name)) return false;
      String
       parent_path = parent_name.substring(0,
                             parent_name.lastIndexOf('/'));
      return (child_name.startsWith(parent_path));
  }


  private void killSnarf() {
       SnarfCount--;
  }

  private void killSnarf(int count) {
        SnarfCount -= count;
  }

  public Snarf(String urlString, TextArea debugLog, String opDir, boolean bImgs) {
    this(urlString, debugLog, opDir, bImgs, true);
  }

  public Snarf(String urlString, TextArea debugLog, String opDir, boolean bImgs, boolean bIfOnlyHTML) {
    SnarfCount ++;
    name = urlString;
    logWindow = debugLog;
    snarfDir = opDir;
    ImagesToo = bImgs;
    OnlyHTML = bIfOnlyHTML;
     
/*******
    if (!name.endsWith(".html")) {
        if (!name.endsWith("/")) name += "/";
        name += "index.html";
    }
********/
    if (OnlyHTML &&(!name.endsWith(".html") && !name.endsWith(".htm"))
       ) {
       logWindow.appendText("Can only process HTML files, not " + name + "\n");
       logWindow.appendText("Try appending index.html, default.html or home.html\n");
       logWindow.appendText("Also try .htm instead of .html\n\n");
       return;
    }
    try {
       root = new URL(name);
    } catch (MalformedURLException e) {
        System.err.println("Problem with URL " + name +
             '(' + e.getMessage() + ')');
        return;
    }

    Document d = new Document();
    logWindow.appendText("Contacting " + name + " ... " + "\n");
    try {
        InputStream urlStream = root.openStream();

        // logWindow.appendText("Done!" + "\n");

        new Parser(urlStream, d);
        urlStream.close();

    } catch (IOException e) {
        System.err.println("IO Error (" + e + ")" + root.toExternalForm());
        return;
    }

    directRefs = new Vector(16, 16);

    logWindow.appendText("Document Title: " + d.getTitle() + "\n");

    for (Enumeration tags = d.getTags().elements();
               tags.hasMoreElements(); ) {
       TagRef tagRef = (TagRef) tags.nextElement();
       if (ShouldSnarf(tagRef)) directRefs.addElement(tagRef);
    }
  } // End Snarf(String)

  // Debugging routine
  public void PrintRefs() {
      if (directRefs == null) {
         return;
      }
      for (Enumeration l = directRefs.elements();
            l.hasMoreElements();
          ) {
        TagRef t = (TagRef) l.nextElement();
        logWindow.appendText("Ref: " + t.getParameter("href") + "\n");
      }
  }

  // Debugging routine
  public void PrintVisited() {
      if (visitedURLs == null) {
         return;
      }
      for (Enumeration l = visitedURLs.keys();
            l.hasMoreElements();
          ) {
        URL url = (URL) l.nextElement();
        logWindow.appendText(url.toExternalForm() + "\n");
      }
  }

  // This is the routine that does all the work ... it descends
  // down the list of tags referenced (directly) by the root doc
  // and decides whether to grab the docs that are in that list
  // and also grabs other docs (e.g. images) that need to be grabbed.

  public void Flatten() {
      if (directRefs == null) {
         return;
      }

      visit(root); // process this doc

      if (SnarfCount > maxSnarfCount) {
        logWindow.appendText("Snarfed too many docs (" + SnarfCount + ")\n");
        return;
      }

      // Go for the next level

      int linkCount = 0;
      for (Enumeration l = directRefs.elements();
           l.hasMoreElements();
          ) {
          // Consider each direct ref
          TagRef t = (TagRef) l.nextElement();
          URL linkURL;
          if (t.tag.name.equals("img")) {
              String link = t.getParameter("src");

              try {
                linkURL = new URL(root, link);
                if (root.getHost().equals(linkURL.getHost()) &&
                    root.getProtocol().equals(linkURL.getProtocol()) &&
                    IsDirectDescendant(root, linkURL) &&
                    !visitedURLs.containsKey(linkURL)) {
                    linkCount ++;
                    visit (linkURL);
                }
              }   catch (MalformedURLException e) {
                 //System.err.println("Problem with link " +
                 //    link;
              }

           } else {
               String link = NoLabel(t.getParameter("href"));

               try {
                   linkURL = new URL(root, link);
                   if (root.getHost().equals(linkURL.getHost()) &&
                      root.getProtocol().equals(linkURL.getProtocol()) &&
                      IsDirectDescendant(root, linkURL) &&
                      !visitedURLs.containsKey(linkURL)) {
        
                      linkCount ++;
                      Snarf s = new Snarf(linkURL.toExternalForm(), logWindow, snarfDir, ImagesToo);
                      s.Flatten();
                   }
                } catch (MalformedURLException e) {
                   //System.err.println("Problem with link " +
                   //    link;
                }


           }
      }
      killSnarf(linkCount); // hopefully they have been gc'd
  }

   private void CopyURL(URL url, File of) {

      byte buffer[] = new byte [1024*16];
      try {
         InputStream is = url.openStream();
         FileOutputStream fos = new FileOutputStream(of);
         while (true) {
           int len = is.read(buffer);
           if (len < 0) break;
           fos.write(buffer, 0, len);
         }
         fos.close();
         is.close();
      } catch (IOException e) {}
   }

   public void visit (URL url) {
      
      visitedURLs.put(url, new Integer(1));

      File f = new File(snarfDir + File.separator + url.getFile().replace('/',File.separatorChar));
      File d = new File(f.getParent());
      // logWindow.appendText("Snarfing " + f + "\n");
      // System.out.println("Creating dirs for " + f + " and path " + d);
      d.mkdirs(); // ignore the info about whther any were made
      CopyURL(url, f);
   }

}

